/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.didacticunit

import android.app.Activity
import android.app.Dialog
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import com.jbo.vga.R
import com.jbo.vga.extensions.levelColor
import com.jbo.vga.extensions.levelDarkColor
import com.jbo.vga.extensions.setProgressDialog
import com.jbo.vga.model.Page
import com.jbo.vga.model.RemoteDataSource
import com.jbo.vga.model.RemoteFileSource
import com.jbo.vga.page.PageActivity


class DidacticUnitActivity : AppCompatActivity(), DidacticUnitContract.ViewInterface {

    private lateinit var didacticUnitPresenter: DidacticUnitContract.PresenterInterface
    private lateinit var dialog: Dialog
    private lateinit var level: String
    private var didacticUnitNumber = -1
    private var pages: List<Page> = emptyList()
    private var pageNumber = 0
    private var navigationDirection = NAVIGATION_DIRECTION_FORWARD

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_didactic_unit)

        // Retrieve intent extras
        level = intent.getStringExtra(ARG_LEVEL).toString()
        if (level=="") {
            errorLoadingPage()
        }

        didacticUnitNumber = intent.getIntExtra(ARG_DIDACTIC_UNIT_NUMBER, -1)
        if (didacticUnitNumber==-1) {
            errorLoadingPage()
        } else {
            didacticUnitNumber++
        }

        val requiresResources = intent.getBooleanExtra(ARG_DIDACTIC_UNIT_REQUIRES_RESOURCES, false)

        initPresenter()

        initStatusBar(level)
            dialog = setProgressDialog(this, levelColor(level), getString(R.string.loading_didactic_unit)
        )

        if (requiresResources) {
            didacticUnitPresenter.synchronizeResources(level, didacticUnitNumber, cacheDir)
        } else {
            didacticUnitPresenter.loadDidacticUnit(level, didacticUnitNumber)
        }

    }

    private fun initPresenter() {
        val remoteDataSource = RemoteDataSource()
        val remoteFileSource = RemoteFileSource()
        didacticUnitPresenter = DidacticUnitPresenter(this, remoteDataSource, remoteFileSource)
    }

    private fun initStatusBar(level: String) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            window.statusBarColor = ContextCompat.getColor(this, levelDarkColor(level))
        }
    }

    private fun openPage(page: Page) {

        val intent = Intent(this, PageActivity::class.java)
        intent.putExtra(ARG_PAGE, page)
        intent.putExtra(ARG_LEVEL, level)
        intent.putExtra(ARG_DIDACTIC_UNIT_NUMBER, didacticUnitNumber)

        startActivityForResult(intent, REQUEST_CODE)
        when (navigationDirection) {
            NAVIGATION_DIRECTION_BACKWARD -> overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left)
            else -> overridePendingTransition(R.anim.slide_in_left, R.anim.slide_out_right)
        }
    }

    private fun errorLoadingPage() {
        Toast.makeText(this, R.string.error_loading_page, Toast.LENGTH_SHORT).show()
        finish() // Return to index
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)

        if (requestCode == REQUEST_CODE && resultCode == Activity.RESULT_OK && data != null) {
            if (data.hasExtra(ARG_PAGE_NAVIGATION_DIRECTION)) {
                navigationDirection = data.getStringExtra(ARG_PAGE_NAVIGATION_DIRECTION).toString()
                when (navigationDirection) {
                    NAVIGATION_DIRECTION_BACKWARD -> pageNumber--
                    NAVIGATION_DIRECTION_FORWARD -> pageNumber++
                    NAVIGATION_TO_INDEX -> pageNumber = -1 // Return to index
                }
            }
        } else {
            errorLoadingPage()
        }

        // Open page or return to index
        if (pageNumber >= 0 && pageNumber<pages.size) {
            openPage(pages[pageNumber])
        }
        else {
            finish()
        }
    }

    override fun openDidacticUnit(pages: List<Page>) {
        this.pages = pages
        openPage(pages[pageNumber])
    }

    override fun showProgress() {
        dialog.show()
    }

    override fun hideProgress() {
        dialog.dismiss()
    }

    override fun didacticUnitSynchronizationResult(synchronized: Boolean) {
        if (synchronized) {
            Toast.makeText(this, R.string.didactic_unit_synchronized, Toast.LENGTH_SHORT).show()
        }

        if (didacticUnitPresenter.didacticUnitReady(cacheDir, level, didacticUnitNumber.toString())) {
            didacticUnitPresenter.loadDidacticUnit(level, didacticUnitNumber)
        } else {
            Toast.makeText(this, R.string.didactic_unit_not_downloaded, Toast.LENGTH_LONG).show()
            finish()
        }
    }

    override fun displayError(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show()
    }

    override fun onBackPressed() {
        finish()
    }

    override fun finish() {
        super.finish()
        if (pageNumber>=pages.size) {
            overridePendingTransition(R.anim.slide_in_left, R.anim.slide_out_right)
        } else {
            overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left)
        }
    }

    override fun onStop() {
        super.onStop()
        didacticUnitPresenter.stop()
    }

    companion object {
        const val ARG_LEVEL = "level"
        const val ARG_DIDACTIC_UNIT_NUMBER = "didactic_unit_number"
        const val ARG_DIDACTIC_UNIT_REQUIRES_RESOURCES = "didactic_unit_requires_resources"
        const val ARG_PAGE_NAVIGATION_DIRECTION = "page_navigation_direction"
        const val ARG_PAGE = "page"

        const val REQUEST_CODE = 1

        const val NAVIGATION_DIRECTION_BACKWARD = "b"
        const val NAVIGATION_DIRECTION_FORWARD = "f"
        const val NAVIGATION_TO_INDEX = "i"

    }

}
