/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.didacticunit

import com.jbo.vga.extensions.*
import com.jbo.vga.model.Page
import com.jbo.vga.model.RemoteDataSource
import com.jbo.vga.model.RemoteFileSource
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.annotations.NonNull
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.observers.DisposableObserver
import io.reactivex.schedulers.Schedulers
import java.io.File

class DidacticUnitPresenter(private var viewInterface: DidacticUnitContract.ViewInterface, private var remoteDataSource: RemoteDataSource, private var remoteFileSource: RemoteFileSource)  :
    DidacticUnitContract.PresenterInterface {

    private val compositeDisposable = CompositeDisposable()
    private var didacticUnitResourceList = emptyList<String>()
    private var didacticUnitRemotePath : String = ""
    private var didacticUnitCachePath: String = ""
    private var resourceCount = 0
    private var synchronizedResources = 0

    /// CALL RESULTS ///
    private val synchronizeResourcesObservable: (String) -> Observable<List<String>> get() = { didacticUnitRemotePath -> remoteFileSource.getDidacticUnitResourcesList(didacticUnitRemotePath) }
    private val synchronizeResourcesObserver: DisposableObserver<List<String>>
        get() = object : DisposableObserver<List<String>>() {

            override fun onNext(didacticUnitResources: List<String>) {
                if (didacticUnitResources.isEmpty()) {
                    viewInterface.hideProgress()
                    viewInterface.didacticUnitSynchronizationResult(false)
                } else {
                    // Resources returned -> delete possible previous download completed file
                    didacticUnitDeleteDownloadCompletedFile(didacticUnitCachePath)

                    didacticUnitResourceList = didacticUnitResources
                    resourceCount = didacticUnitResources.size
                    for (resourceName in didacticUnitResources) {
                        synchronizeResource(resourceName)
                    }
                }
            }
            override fun onError(@NonNull e: Throwable) {
                viewInterface.hideProgress()
                //viewInterface.displayError(e.message.toString())
                viewInterface.didacticUnitSynchronizationResult(false)
            }
            override fun onComplete() {
                //
            }
        }

    private val synchronizeResourceObservable: (String, String, String) -> Observable<Boolean> get() = { resourceName, didacticUnitCachePath, didacticUnitRemotePath -> remoteFileSource.synchronizeResource(resourceName, didacticUnitCachePath, didacticUnitRemotePath) }
    private val synchronizeResourceObserver: DisposableObserver<Boolean>
        get() = object : DisposableObserver<Boolean>() {

            override fun onNext(synchronized: Boolean) {
                if (synchronized) {
                    synchronizedResources++
                }
                if (synchronizedResources == resourceCount) {
                    postSynchronizeResources()
                }
            }
            override fun onError(@NonNull e: Throwable) {
                viewInterface.hideProgress()
                viewInterface.displayError(e.message.toString())
            }
            override fun onComplete() {
                //
            }
        }

    private val postSynchronizeResourcesObservable: (String, List<String>) -> Observable<Boolean> get() = { didacticUnitCachePath, didacticUnitResourceList -> remoteFileSource.postSynchronizeResources(didacticUnitCachePath, didacticUnitResourceList) }
    private val postSynchronizeResourcesObserver: DisposableObserver<Boolean>
        get() = object : DisposableObserver<Boolean>() {

            override fun onNext(synchronized: Boolean) {
                viewInterface.hideProgress()
                viewInterface.didacticUnitSynchronizationResult(synchronized)
            }
            override fun onError(@NonNull e: Throwable) {
                viewInterface.hideProgress()
            }
            override fun onComplete() {
                //
            }
        }

    private val loadDidacticUnitObservable: (String, Int) -> Observable<List<Page>> get() = { level, didacticUnitNumber -> remoteDataSource.getDidacticUnitPages(level, didacticUnitNumber) }
    private val loadDidacticUnitObserver: DisposableObserver<List<Page>>
    get() = object : DisposableObserver<List<Page>>() {

        override fun onNext(pages: List<Page>) {
            viewInterface.openDidacticUnit(pages)
        }
        override fun onError(@NonNull e: Throwable) {
            viewInterface.displayError(e.message.toString())
        }
        override fun onComplete() {
            //
        }
    }

    /// CALLS ///
    override fun synchronizeResources(level: String, didacticUnitNumber: Int, cacheDirectory: File) {

        didacticUnitCachePath = buildCachePath(cacheDirectory, level, didacticUnitNumber.toString())
        createDidacticUnitDirectory(didacticUnitCachePath)
        didacticUnitRemotePath = buildDidacticUnitRemotePath(level, didacticUnitNumber.toString())

        val synchronizeResourcesDisposable = synchronizeResourcesObservable(didacticUnitRemotePath)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(synchronizeResourcesObserver)

        compositeDisposable.add(synchronizeResourcesDisposable)
        viewInterface.showProgress()
    }

    fun synchronizeResource(resourceName: String) {
        val synchronizeResourceDisposable = synchronizeResourceObservable(resourceName, didacticUnitCachePath, didacticUnitRemotePath)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(synchronizeResourceObserver)

        compositeDisposable.add(synchronizeResourceDisposable)
    }

    fun postSynchronizeResources() {
        val postSynchronizeResourceDisposable = postSynchronizeResourcesObservable(didacticUnitCachePath, didacticUnitResourceList)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(postSynchronizeResourcesObserver)

        compositeDisposable.add(postSynchronizeResourceDisposable)
    }

    override fun loadDidacticUnit(level: String, didacticUnitNumber: Int) {

        val loadDidacticUnitDisposable = loadDidacticUnitObservable(level, didacticUnitNumber)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(loadDidacticUnitObserver)

        compositeDisposable.add(loadDidacticUnitDisposable)
    }

    override fun didacticUnitReady(cacheDirectory: File, level:String, didacticUnit: String): Boolean {
        return didacticUnitDownloaded(cacheDirectory, level, didacticUnit)
    }

    override fun stop() {
        compositeDisposable.clear()
    }

}