/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.home

import android.content.Intent
import android.os.Bundle
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.jbo.vga.R
import com.jbo.vga.credits.CreditsActivity
import com.jbo.vga.index.IndexActivity
import com.jbo.vga.login.LoginActivity
import com.jbo.vga.model.RemoteDataSource
import com.mikepenz.materialdrawer.AccountHeader
import com.mikepenz.materialdrawer.AccountHeaderBuilder
import com.mikepenz.materialdrawer.Drawer
import com.mikepenz.materialdrawer.DrawerBuilder
import com.mikepenz.materialdrawer.model.DividerDrawerItem
import com.mikepenz.materialdrawer.model.PrimaryDrawerItem
import com.mikepenz.materialdrawer.model.ProfileDrawerItem
import com.mikepenz.materialdrawer.model.interfaces.IDrawerItem
import com.mikepenz.materialdrawer.model.interfaces.IProfile
import kotlinx.android.synthetic.main.activity_home.*

class HomeActivity : AppCompatActivity(), HomeContract.ViewInterface {

    private lateinit var homePresenter: HomeContract.PresenterInterface
    private lateinit var currentUserEmail: String

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_home)

        initBackground()

        initPresenter()

        currentUserEmail = homePresenter.getCurrentUserEmail()

        initDrawer()

        // Load all level indexes into Firebase cache
        homePresenter.cacheIndexes()


        // A1
        ibA1.setOnClickListener {
            startLevel(LEVEL_A1)
        }

        // A2
        ibA2.setOnClickListener {
            startLevel(LEVEL_A2)
        }

        // B1
        ibB1.setOnClickListener {
            startLevel(LEVEL_B1)
        }

        // B2
        ibB2.setOnClickListener {
            startLevel(LEVEL_B2)
        }

    }

    private fun initBackground() {
        //  Because of compatibility reasons, background image initialized programmatically
        val background = VectorDrawableCompat.create(resources, R.drawable.background_home, null)
        clHome.background = background
    }

    private fun initDrawer() {

        //Items
        val item1 = PrimaryDrawerItem().withIdentifier(CREDITS.toLong()).withName(R.string.drawer_menu_credits)
        val item2 = PrimaryDrawerItem().withIdentifier(LOGOUT.toLong()).withName(R.string.drawer_menu_logout)

        // Create the AccountHeader

        val headerResult = AccountHeaderBuilder()
            .withActivity(this)
            //.withHeaderBackground(R.color.primary_light)
            .addProfiles(
                ProfileDrawerItem().withName(R.string.app_name).withEmail(currentUserEmail)
                    .withIcon(VectorDrawableCompat.create(resources, R.drawable.logo, null))                )
            .withOnAccountHeaderListener(object : AccountHeader.OnAccountHeaderListener {
                override fun onProfileChanged(view: View?, profile: IProfile<*>, current: Boolean): Boolean {
                    return false
                }
            })
            .build()

        //create the drawer and remember the `Drawer` result object
        val menuDrawer = DrawerBuilder()
            .withActivity(this)
            .withToolbar(tbHome)
            .withAccountHeader(headerResult)
            .addDrawerItems(
                item1,
                DividerDrawerItem(),
                item2
            )
            .withOnDrawerItemClickListener(object : Drawer.OnDrawerItemClickListener {
                override fun onItemClick(view: View?, position: Int, drawerItem: IDrawerItem<*>): Boolean {
                    when (position) {
                        CREDITS -> showCredits()
                        LOGOUT -> logout()
                        else -> {
                        }
                    }
                    return false
                }
            })
            .build()

        menuDrawer.actionBarDrawerToggle?.drawerArrowDrawable?.color = ContextCompat.getColor(this, R.color.colorWhite)

        menuDrawer.setSelection(-1)
    }

    private fun initPresenter(){
        val remoteDataSource = RemoteDataSource()
        homePresenter = HomePresenter(this, remoteDataSource)
    }

    private fun startLevel(level: String) {
        val intent = Intent(this, IndexActivity::class.java)
        intent.putExtra(IndexActivity.ARG_LEVEL, level)
        startActivity(intent)
        overridePendingTransition(R.anim.slide_in_left, R.anim.slide_out_right)
    }

    private fun showCredits() {
        val intent = Intent(this, CreditsActivity::class.java)
        startActivity(intent)
        overridePendingTransition(R.anim.slide_in_left, R.anim.slide_out_right)
    }

    private fun logout() {
        homePresenter.logout()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
            else -> false
        }
    }

    override fun loggedOut() {
        val intent = Intent(this, LoginActivity::class.java)
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK)
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        startActivity(intent)
        overridePendingTransition(R.anim.slide_in_left, R.anim.slide_out_right)
    }

    override fun logoutFailed() {
        Toast.makeText(this, R.string.logout_failed, Toast.LENGTH_SHORT).show()
    }

    override fun finish() {
        super.finish()
        overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left)
    }

    override fun onStop() {
        super.onStop()
        homePresenter.stop()
    }

    companion object {
        // DrawerMenu options
        private const val CREDITS = 1
        private const val LOGOUT = 3

        const val LEVEL_A1 = "a1"
        const val LEVEL_A2 = "a2"
        const val LEVEL_B1 = "b1"
        const val LEVEL_B2 = "b2"

    }

}