/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.index

import android.app.Dialog
import android.os.Build
import android.os.Bundle
import android.view.MenuItem
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.jbo.vga.R
import com.jbo.vga.adapters.IndexRecyclerViewAdapter
import com.jbo.vga.didacticunit.DidacticUnitActivity
import com.jbo.vga.extensions.levelColor
import com.jbo.vga.extensions.levelDarkColor
import com.jbo.vga.extensions.levelIcon
import com.jbo.vga.extensions.setProgressDialog
import com.jbo.vga.model.DidacticUnit
import com.jbo.vga.model.RemoteDataSource
import kotlinx.android.synthetic.main.activity_index.*
import kotlinx.android.synthetic.main.toolbar_index.*
import java.util.*

class IndexActivity : AppCompatActivity(), IndexContract.ViewInterface {

    private lateinit var indexPresenter: IndexContract.PresenterInterface
    private lateinit var dialog: Dialog
    private lateinit var level: String
    private var adapter: IndexRecyclerViewAdapter? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_index)

        // Retrieve intent extras
        level = intent.getStringExtra(DidacticUnitActivity.ARG_LEVEL).toString()

        initStatusBar()

        initToolbar()

        initPresenter()

        initRecyclerView()

        dialog = setProgressDialog(this, levelColor(level), getString(R.string.loading_index))

        indexPresenter.getLevelIndex(level)
    }

    private fun initStatusBar() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            window.statusBarColor = ContextCompat.getColor(this, levelDarkColor(level))
        }
    }

    private fun initToolbar() {
        tbIndex.setBackgroundColor(ContextCompat.getColor(this, levelColor(level)))

        // Because of compatibility reasons, initialize icons programmatically
        tbIndex.navigationIcon = VectorDrawableCompat.create(resources, R.drawable.ic_navigate_before, null)
        tbIndex.logo = VectorDrawableCompat.create(resources, levelIcon(level), null)
        setSupportActionBar(tbIndex)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
    }

    private fun initPresenter(){
        val remoteDataSource = RemoteDataSource()
        indexPresenter = IndexPresenter(this, remoteDataSource)
    }

    private fun initRecyclerView(){
        rvIndex.addItemDecoration(DividerItemDecoration(this, LinearLayoutManager.VERTICAL))
        val layoutManager = LinearLayoutManager(this)
        rvIndex.layoutManager = layoutManager
        adapter = IndexRecyclerViewAdapter(level, ArrayList())
        rvIndex.adapter = adapter
    }

    override fun displayIndex(index: List<DidacticUnit>){
        adapter?.setIndex(index)
    }

    override fun showProgress() {
        dialog.show()
    }

    override fun hideProgress() {
        dialog.dismiss()
    }

    override fun displayError(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
            else -> false
        }
    }

    override fun onResume() {
        super.onResume()
        adapter?.setCanStartDidacticUnit(true)
    }

    override fun finish() {
        super.finish()
        overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left)
    }

    override fun onStop() {
        super.onStop()
        indexPresenter.stop()
    }

    companion object {
        const val ARG_LEVEL = "level"
    }

}