/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.login

import com.jbo.vga.model.RemoteDataSource
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.annotations.NonNull
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.observers.DisposableObserver
import io.reactivex.schedulers.Schedulers

class LoginPresenter(private var viewInterface: LoginContract.ViewInterface, private var remoteDataSource: RemoteDataSource) : LoginContract.PresenterInterface {

    private val compositeDisposable = CompositeDisposable()

    /// CALL RESULTS ///
    private val loginObservable: (String, String) -> Observable<Boolean> = { email, password -> remoteDataSource.login(email, password) }
    private val loginObserver: DisposableObserver<Boolean>
        get() = object : DisposableObserver<Boolean>() {

            override fun onNext(authenticated: Boolean) {
                if (authenticated) {
                    viewInterface.hideProgress()
                    viewInterface.loginSuccessfully()
                }
            }
            override fun onError(@NonNull e: Throwable) {
                viewInterface.hideProgress()
                viewInterface.displayError(e.message.toString())
            }
            override fun onComplete() {
                //
            }
        }

    private val sendPasswordResetEmailObservable: (String) -> Observable<Boolean> = { email -> remoteDataSource.sendPasswordResetEmail(email) }
    private val sendPasswordResetEmailObserver: DisposableObserver<Boolean>
        get() = object : DisposableObserver<Boolean>() {

            override fun onNext(authenticated: Boolean) {
                if (authenticated) {
                    viewInterface.hideProgress()
                    viewInterface.passwordResetEmailSendSuccessfully()
                }
            }
            override fun onError(@NonNull e: Throwable) {
                viewInterface.hideProgress()
                viewInterface.displayError(e.message.toString())
            }
            override fun onComplete() {
                //
            }
        }

    /// CALLS ///
    override fun stillAuthenticated(): Boolean {
        return remoteDataSource.stillAuthenticated()
    }

    override fun login(email: String, password: String) {
        val loginDisposable = loginObservable(email, password)
            // Subscribe on a background thread
            .subscribeOn(Schedulers.io())
            // Observe on the main thread
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(loginObserver)

        compositeDisposable.add(loginDisposable)
        viewInterface.showProgress()
    }


    override fun sendPasswordResetEmail(email: String) {
        val sendPasswordResetEmailDisposable = sendPasswordResetEmailObservable(email)
            // Subscribe on a background thread
            .subscribeOn(Schedulers.io())
            // Observe on the main thread
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(sendPasswordResetEmailObserver)

        compositeDisposable.add(sendPasswordResetEmailDisposable)
        viewInterface.showProgress()
    }

    override fun stop() {
        compositeDisposable.clear()
    }

}