/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.model

import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.database.*
import io.reactivex.Observable

// open class required for testing
open class RemoteDataSource {

    private val auth: FirebaseAuth = FirebaseAuth.getInstance()
    private val database = FirebaseDatabase.getInstance()

    open fun createAccount(email: String, password: String): Observable<Boolean> {
        return Observable.create { emitter ->
            auth.createUserWithEmailAndPassword(email, password)
                .addOnCompleteListener { task ->
                    if (task.isSuccessful) {
                        emitter.onNext(true)
                    } else {
                        emitter.onError(RuntimeException(task.exception))
                    }
                    emitter.onComplete()
                }
        }
    }

    open fun login(email: String, password: String): Observable<Boolean> {
        return Observable.create { emitter ->
        auth.signInWithEmailAndPassword(email, password)
            .addOnCompleteListener { task ->
                if (task.isSuccessful) {
                    emitter.onNext(true)
                } else {
                    emitter.onError(RuntimeException(task.exception))
                }
                emitter.onComplete()
            }
        }
    }

    open fun stillAuthenticated(): Boolean {
        return FirebaseAuth.getInstance().currentUser!=null
    }

    open fun logout(): Observable<Boolean> {
        return Observable.create { emitter ->
            auth.signOut()
            emitter.onNext(!stillAuthenticated())
            emitter.onComplete()
        }
    }

    open fun sendPasswordResetEmail(email: String): Observable<Boolean> {
        return Observable.create { emitter ->
            auth.sendPasswordResetEmail(email)
                .addOnCompleteListener { task ->
                    if (task.isSuccessful) {
                        emitter.onNext(true)
                    } else {
                        emitter.onError(RuntimeException(task.exception))
                    }
                    emitter.onComplete()
                }
        }
    }

    open fun getCurrentUserEmail(): String {
       return FirebaseAuth.getInstance().currentUser?.email.toString()
    }

    open fun getCredits(): Observable<List<Credit>> {
        return Observable.create { emitter ->
            val listener: ValueEventListener = object : ValueEventListener {
                override fun onDataChange(dataSnapshot: DataSnapshot) {
                    if (dataSnapshot.childrenCount > 0) {
                        val t: GenericTypeIndicator<List<Credit>> = object : GenericTypeIndicator<List<Credit>>() {}
                        val credits: List<Credit> = dataSnapshot.getValue(t)!!
                        emitter.onNext(credits)
                    }
                }

                override fun onCancelled(databaseError: DatabaseError) {
                    if (!emitter.isDisposed) {
                        emitter.onError(databaseError.toException())
                    }
                }
            }
            database.reference.child("credits").addValueEventListener(listener)
        }
    }

    open fun cacheIndexes(): Observable<Boolean> {
        return Observable.create { emitter ->
            val listener: ValueEventListener = object : ValueEventListener {
                override fun onDataChange(dataSnapshot: DataSnapshot) {
                    if (dataSnapshot.childrenCount > 0) {
                        emitter.onNext(true)
                    }
                }

                override fun onCancelled(databaseError: DatabaseError) {
                    if (!emitter.isDisposed) {
                        emitter.onError(databaseError.toException())
                    }
                }
            }
            database.reference.child("indexes").addValueEventListener(listener)
        }
    }

    open fun getLevelIndex(level: String): Observable<List<DidacticUnit>> {
        return Observable.create { emitter ->
            val listener: ValueEventListener = object : ValueEventListener {
                override fun onDataChange(dataSnapshot: DataSnapshot) {
                    if (dataSnapshot.childrenCount > 0) {
                        val t: GenericTypeIndicator<List<DidacticUnit>> = object : GenericTypeIndicator<List<DidacticUnit>>() {}
                        val didacticUnits: List<DidacticUnit> = dataSnapshot.getValue(t)!!
                        emitter.onNext(didacticUnits)
                    }
                }

                override fun onCancelled(databaseError: DatabaseError) {
                    if (!emitter.isDisposed) {
                        emitter.onError(databaseError.toException())
                    }
                }
            }
            database.reference.child("indexes").child("levels").child(level).child("didacticUnits")
                .addValueEventListener(listener)
        }
    }

    open fun getDidacticUnitPages(level: String, didacticUnitNumber: Int): Observable<List<Page>> {
        return Observable.create { emitter ->
            val listener: ValueEventListener = object : ValueEventListener {
                override fun onDataChange(dataSnapshot: DataSnapshot) {
                    if (dataSnapshot.childrenCount > 0) {
                        val t: GenericTypeIndicator<List<Page>> =
                            object : GenericTypeIndicator<List<Page>>() {}
                        val pages: List<Page> = dataSnapshot.getValue(t)!!
                        emitter.onNext(pages)
                    }
                }

                override fun onCancelled(databaseError: DatabaseError) {
                    if (!emitter.isDisposed) {
                        emitter.onError(databaseError.toException())
                    }
                }
            }
            // didacticUnit number correction (in database -1):
            val fbDidacticUnitNumber = didacticUnitNumber -1
            database.reference.child("stuff").child("levels").child(level).child("didacticUnits").child(fbDidacticUnitNumber.toString()).child("pages")
                .addValueEventListener(listener)
        }
    }

}