/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.model

import com.google.firebase.ktx.Firebase
import com.google.firebase.storage.ktx.storage
import com.jbo.vga.extensions.didacticUnitDownloadCompletedFile
import io.reactivex.Observable
import java.io.File

open class RemoteFileSource {

    private val storageReference = Firebase.storage.reference

    open fun getDidacticUnitResourcesList(didacticUnitRemotePath: String): Observable<List<String>> {
        return Observable.create { emitter ->
            storageReference.child(didacticUnitRemotePath).listAll().addOnSuccessListener { listResult ->
                emitter.onNext(listResult.items.map { value -> value.name })
            }.addOnFailureListener { exception ->
                if (!emitter.isDisposed) {
                    emitter.onError(exception)
                }
            }
        }
    }

    open fun synchronizeResource(resourceName: String, cachePath: String, remotePath: String): Observable<Boolean> {
        return Observable.create { emitter ->
            val file = File(cachePath, resourceName)
            if (!file.exists()) {
                storageReference.child("$remotePath/$resourceName").getFile(file)
                    .addOnCompleteListener { task ->
                        emitter.onNext(task.isSuccessful)
                    }
            } else { // local file exists -> verify validity
                storageReference.child("$remotePath/$resourceName").metadata.addOnSuccessListener { metadata ->
                    // Local file still valid?
                    if (metadata.updatedTimeMillis > file.lastModified() || metadata.sizeBytes != file.length()) {
                        // Last update date or size differ -> download
                        storageReference.child("$remotePath/$resourceName").getFile(file)
                            .addOnCompleteListener { task ->
                                emitter.onNext(task.isSuccessful)
                            }
                    } else {
                        emitter.onNext(true)
                    }
                }.addOnFailureListener {
                    emitter.onNext(false)
                }
            }
        }
    }

    open fun postSynchronizeResources(cachePath: String, resourceList: List<String>): Observable<Boolean> {
        return Observable.create { emitter ->
            // Delete local files that no longer exists in the remote storage
            val dir = File(cachePath)
            dir.walkTopDown().forEach {
                if (it.isFile && !resourceList.contains(it.name))
                    it.delete()
            }

            // Create dummy file for didactic unit download completed check
            val didacticUnitDownloadCompleted = didacticUnitDownloadCompletedFile(cachePath)
            didacticUnitDownloadCompleted.createNewFile()

            emitter.onNext(true)
        }
    }

}