/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.page

import android.app.Activity
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.view.MenuItem
import android.view.View
import android.widget.ImageView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.vectordrawable.graphics.drawable.VectorDrawableCompat
import com.github.twocoffeesoneteam.glidetovectoryou.GlideToVectorYou
import com.jbo.vga.R
import com.jbo.vga.didacticunit.DidacticUnitActivity
import com.jbo.vga.extensions.hideSoftKeyboard
import com.jbo.vga.extensions.levelColor
import com.jbo.vga.extensions.levelDarkColor
import com.jbo.vga.model.Page
import com.jbo.vga.page.fragments.audiocontrol.AudioControlFragment
import com.jbo.vga.page.fragments.correctoptions.CorrectOptionsFragment
import com.jbo.vga.page.fragments.listendynamicoptions.ListenDynamicOptionsFragment
import com.jbo.vga.page.fragments.listenstaticoptions.ListenStaticOptionsFragment
import com.jbo.vga.page.fragments.oralcomprehension.OralComprehensionFragment
import com.jbo.vga.page.fragments.readingcomprehension.ReadingComprehensionFragment
import com.jbo.vga.page.fragments.textwriting.TextWritingFragment
import com.jbo.vga.page.fragments.theory.TheoryImageFragment
import kotlinx.android.synthetic.main.activity_page.*
import kotlinx.android.synthetic.main.include_statement.*
import kotlinx.android.synthetic.main.toolbar_page.*
import java.util.*

class PageActivity : AppCompatActivity(), PageListener {

    private lateinit var pagePresenter: PageContract.PresenterInterface
    private lateinit var page: Page
    private lateinit var level: String
    private var didacticUnitNumber: Int = -1

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_page)

        // Retrieve intent extras
        level = intent.getStringExtra(DidacticUnitActivity.ARG_LEVEL).toString()
        didacticUnitNumber = intent.getIntExtra(DidacticUnitActivity.ARG_DIDACTIC_UNIT_NUMBER, -1)

        // Retrieve page
        page = intent.getSerializableExtra(DidacticUnitActivity.ARG_PAGE) as Page

        initStatusBar(level)

        initToolbar(level, page)

        initPresenter()

        initStatement()

        if (savedInstanceState == null) {
            initContent()
        }

        // Set focus change listener
        llPage.onFocusChangeListener = View.OnFocusChangeListener { it, focused ->
            if (focused) {
                // Hide soft keyboard when edit text lost focus
                hideSoftKeyboard(it)
            }
        }

        ibIndex.setOnClickListener {
            intent.putExtra(
                DidacticUnitActivity.ARG_PAGE_NAVIGATION_DIRECTION,
                DidacticUnitActivity.NAVIGATION_TO_INDEX
            )
            finish()
        }

        ibNavigationNext.setOnClickListener {
            intent.putExtra(
                DidacticUnitActivity.ARG_PAGE_NAVIGATION_DIRECTION,
                DidacticUnitActivity.NAVIGATION_DIRECTION_FORWARD
            )
            finish()
        }
    }

    private fun initStatusBar(level: String) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            window.statusBarColor = ContextCompat.getColor(this, levelDarkColor(level))
        }
    }

    private fun initToolbar(level: String, page: Page) {

        tbPage.setBackgroundColor(ContextCompat.getColor(this, levelColor(level)))
        tbPage.title = page.titleDE
        tbPage.subtitle = page.titleES

        // Because of compatibility reasons, initialize icons programmatically
        tbPage.navigationIcon = VectorDrawableCompat.create(resources, R.drawable.ic_navigate_before,null)
        var icon = VectorDrawableCompat.create(resources, R.drawable.ic_menu, null)
        ibIndex.background = icon
        icon = VectorDrawableCompat.create(resources, R.drawable.ic_navigate_next, null)
        ibNavigationNext.background = icon

        setSupportActionBar(tbPage)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
    }

    private fun initPresenter() {
        pagePresenter = PagePresenter()
    }

    private fun initStatement() {

        if (page.statementDE.isEmpty() && page.statementES.isEmpty()) {
            llStatement.visibility = View.GONE
            return
        }

        if (page.statementDE.isEmpty()) {
            tvStatementDe.visibility = View.GONE
        } else {
            tvStatementDe.text = page.statementDE
        }

        if (page.statementES.isEmpty()) {
            tvStatementEs.visibility = View.GONE
        } else {
            tvStatementEs.text = page.statementES
        }
    }

    private fun initContent() {

        var fragmentInstance: Fragment ?= null
        when (page.exerciseType) {
            EXERCISE_TYPE_CORRECT_OPTIONS -> fragmentInstance = CorrectOptionsFragment.newInstance(level, page)
            EXERCISE_TYPE_LISTEN_DYNAMIC_OPTIONS -> fragmentInstance = ListenDynamicOptionsFragment.newInstance(level, page)
            EXERCISE_TYPE_LISTEN_STATIC_OPTIONS -> fragmentInstance = ListenStaticOptionsFragment.newInstance(level, page)
            EXERCISE_TYPE_ORAL_COMPREHENSION -> fragmentInstance = OralComprehensionFragment.newInstance(level, page)
            EXERCISE_TYPE_READING_COMPREHENSION -> fragmentInstance = ReadingComprehensionFragment.newInstance(level, page)
            EXERCISE_TYPE_TEXT_WRITING -> fragmentInstance = TextWritingFragment.newInstance(level)
            EXERCISE_TYPE_THEORY_IMAGE -> fragmentInstance = TheoryImageFragment.newInstance(level, page)
        }

        if (fragmentInstance != null) {
            supportFragmentManager.beginTransaction().add(R.id.flFragmentContainer2, fragmentInstance).commit()
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
            else -> false
        }
    }

    override fun onBackPressed() {
        intent.putExtra(
            DidacticUnitActivity.ARG_PAGE_NAVIGATION_DIRECTION,
            DidacticUnitActivity.NAVIGATION_DIRECTION_BACKWARD
        )
        finish()
    }

    override fun finish() {
        setResult(Activity.RESULT_OK, intent)
        super.finish()
    }

    override fun onAttachFragment(fragment: Fragment) {

        when (fragment) {
            is CorrectOptionsFragment -> fragment.setPageListener(this)
            is ListenDynamicOptionsFragment -> fragment.setPageListener(this)
            is ListenStaticOptionsFragment -> fragment.setPageListener(this)
            is OralComprehensionFragment -> fragment.setPageListener(this)
            is ReadingComprehensionFragment -> fragment.setPageListener(this)
            is TextWritingFragment -> fragment.setPageListener(this)
            is TheoryImageFragment -> fragment.setPageListener(this)
        }
    }

    // Page Interface
    override fun getImageUri(resourcePath: String): Uri {
        return pagePresenter.getImageUri(cacheDir, level, didacticUnitNumber, resourcePath)
    }

    override fun getAudioUri(resourcePath: String): Uri {
        return pagePresenter.getAudioUri(cacheDir, level, didacticUnitNumber, resourcePath)
    }

    override fun getFeedbackEmailSubject(level: String): String {
        return "${level.toUpperCase(Locale.ROOT)}: " + tbPage?.title + " - " + tvStatementDe?.text
    }

    override fun loadSvgImage(uri: Uri, imageView: ImageView) {
        GlideToVectorYou.justLoadImage(this, uri, imageView)
    }

    override fun addAudioFragment(level: String, audioPath: String) {
        val fragmentInstance: Fragment = AudioControlFragment.newInstance(level, getAudioUri(audioPath))
        supportFragmentManager.beginTransaction().add(R.id.flFragmentContainer1, fragmentInstance).commit()
    }

    override fun onHideSoftKeyboard(view: View) {
        hideSoftKeyboard(view)
    }

    override fun onContentFinished() {
        intent.putExtra(
            DidacticUnitActivity.ARG_PAGE_NAVIGATION_DIRECTION,
            DidacticUnitActivity.NAVIGATION_DIRECTION_FORWARD
        )
        finish()
    }

    companion object {
        const val EXERCISE_TYPE_CORRECT_OPTIONS = "correct_options"
        const val EXERCISE_TYPE_LISTEN_STATIC_OPTIONS = "listen_static_options"
        const val EXERCISE_TYPE_LISTEN_DYNAMIC_OPTIONS = "listen_dynamic_options"
        const val EXERCISE_TYPE_ORAL_COMPREHENSION = "oral_comprehension"
        const val EXERCISE_TYPE_READING_COMPREHENSION = "reading_comprehension"
        const val EXERCISE_TYPE_TEXT_WRITING = "text_writing"
        const val EXERCISE_TYPE_THEORY_IMAGE = "theory_image"
    }

}