/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.page.fragments.correctoptions

import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.TableLayout
import android.widget.TableRow
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.jbo.vga.R
import com.jbo.vga.extensions.levelColor
import com.jbo.vga.model.Content
import com.jbo.vga.model.Page
import com.jbo.vga.page.PageListener
import kotlinx.android.synthetic.main.fragment_correct_options.view.*

class CorrectOptionsFragment : Fragment() {

    // Page interface
    private lateinit var pageCallback: PageListener
    fun setPageListener(pageCallback: PageListener) {
        this.pageCallback = pageCallback
    }

    private lateinit var level: String
    private lateinit var page: Page

    private lateinit var currentContent : Content
    private var currentContentIndex = -1
    private var selectedOptionsCount = 0
    private var selectedOptions = mutableListOf<Int>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        arguments?.let {
            level = it.getString(ARG_LEVEL).toString()
            page = it.getSerializable(ARG_PAGE) as Page
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_correct_options, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        nextContent(view, true)
    }

    private fun nextContent(view: View, firstQuestion: Boolean) {

        // Disable options to avoid user actions until next content is loaded
        val optionButtons = view.tlOptions.touchables
        for (optionButton in optionButtons) {
            optionButton.isEnabled = false
        }

        val timeUntilNextContent = if (firstQuestion) 0 else TIME_UNTIL_NEXT_CONTENT_ML
        Handler(Looper.getMainLooper()).postDelayed({

            // Delete last content options
            view.tlOptions.removeAllViews()
            selectedOptionsCount = 0
            selectedOptions.clear()

            currentContentIndex++

            // Finished?
            if (currentContentIndex<page.contents.size) {

                // No: load next content
                currentContent = page.contents[currentContentIndex]
                view.tvContent?.text = currentContent.de

                createOptions(view, currentContent)
            } else {
                // Return to index
                pageCallback.onContentFinished()
            }

        }, timeUntilNextContent)

    }

    private fun createOptions(view: View, content: Content) {

        var tableRow: TableRow ?= null
        val actionListener = View.OnClickListener{ selectedOption ->

            if (!selectedOptions.contains(selectedOption.id)) {

                selectedOptionsCount++
                selectedOptions.add(selectedOption.id)

                // Correct answer?
                if (content.correctOptionSequence.contains(selectedOption.id.toString(), ignoreCase = true)) {
                    selectedOption.setBackgroundColor(ContextCompat.getColor(view.context, R.color.correctAnswerColor))
                    nextContent(view, false)
                } else {
                    selectedOption.setBackgroundColor(ContextCompat.getColor(view.context, R.color.incorrectAnswerColor))
                    // Next content?
                    if (selectedOptionsCount == currentContent.options.size) {
                        nextContent(view, false)
                    }
                }
            }
        }

        for ((index, option) in content.options.withIndex()) {

            //  buttons per row
            val newRow = (index % BUTTONS_PER_ROW == 0)
            if (newRow) {
                // Create new row
                tableRow = TableRow(view.context)
                tableRow.layoutParams = TableLayout.LayoutParams(TableLayout.LayoutParams(TableLayout.LayoutParams.MATCH_PARENT, TableLayout.LayoutParams.WRAP_CONTENT))
                tableRow.gravity = Gravity.CENTER_HORIZONTAL
            }

            // Inflate option button used to create the options dynamically
            val buOption =  LayoutInflater.from(view.context).inflate(R.layout.button_option, tableRow, false) as Button
            buOption.id = index + 1 // Adaptation of 0 beginning
            buOption.text = option.de
            buOption.setBackgroundColor(ContextCompat.getColor(view.context, levelColor(level)))
            buOption.setOnClickListener(actionListener)

            tableRow?.addView(buOption)

            // Completed row -> add to main view
            if (newRow) {
                view.tlOptions.addView(tableRow)
            }
        }
    }

    companion object {

        private const val ARG_LEVEL = "level"
        private const val ARG_PAGE = "page"

        const val BUTTONS_PER_ROW = 3
        const val TIME_UNTIL_NEXT_CONTENT_ML: Long = 1000

        @JvmStatic
        fun newInstance(level: String, page: Page) =
            CorrectOptionsFragment().apply {
                arguments = Bundle().apply {

                    putString(ARG_LEVEL, level)
                    putSerializable(ARG_PAGE, page)

                }
            }
    }

}