/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.page.fragments.listendynamicoptions

import android.media.MediaPlayer
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.LinearLayout
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.jbo.vga.R
import com.jbo.vga.extensions.levelColor
import com.jbo.vga.model.Content
import com.jbo.vga.model.Option
import com.jbo.vga.model.Page
import com.jbo.vga.page.PageListener
import kotlinx.android.synthetic.main.fragment_listen_dynamic_options.*
import kotlinx.android.synthetic.main.fragment_listen_dynamic_options.view.*
import kotlin.random.Random

class ListenDynamicOptionsFragment: Fragment() {

    // Page interface
    private lateinit var pageCallback: PageListener
    fun setPageListener(pageCallback: PageListener) {
        this.pageCallback = pageCallback
    }

    private lateinit var level: String
    private lateinit var page: Page

    private lateinit var fragmentView: View
    private lateinit var mpSingle: MediaPlayer
    private lateinit var content: Content
    private lateinit var correctImagePathUri: Uri
    private lateinit var incorrectImagePathUri: Uri
    private lateinit var optionHandler: Handler
    private lateinit var targetHandler: Handler
    private lateinit var optionObject: Runnable
    private lateinit var targetObject: Runnable
    private lateinit var removeHandler: Handler
    private lateinit var removeObject: Runnable

    private var levelColor = R.color.colorDarkGray // default
    private var targetLetter = "a" // default}
    private var visibleOptions = mutableListOf<Option>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        arguments?.let {
            level = it.getString(ARG_LEVEL).toString()
            levelColor = levelColor(level)
            page = it.getSerializable(ARG_PAGE) as Page
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        fragmentView = inflater.inflate(R.layout.fragment_listen_dynamic_options, container, false)

        fragmentView.buStart.setBackgroundColor(ContextCompat.getColor(fragmentView.context, levelColor))
        fragmentView.buStop.setBackgroundColor(ContextCompat.getColor(fragmentView.context, levelColor))

        return fragmentView
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Init stop button
        buStop.setBackgroundColor(ContextCompat.getColor(view.context, R.color.colorDarkGray))
        buStop.isEnabled = false

        // Second content is correct image
        content = page.contents[0]
        correctImagePathUri= pageCallback.getImageUri(content.imagePath)

        // Second content is incorrect image
        content = page.contents[1]
        incorrectImagePathUri = pageCallback.getImageUri(content.imagePath)

        // Dynamic options
        content = page.contents[2]

        // Button listeners
        buStart.setOnClickListener{ start(it) }
        buStop.setOnClickListener{ stop(it) }
    }

    private fun start(view: View) {

        buStart.isEnabled = false
        buStart.setBackgroundColor(ContextCompat.getColor(view.context, R.color.colorDarkGray))
        buStop.setBackgroundColor(ContextCompat.getColor(view.context, levelColor))
        buStop.isEnabled = true

        val actionListener = View.OnClickListener{ selectedOption ->

            val selectedImageView = selectedOption as ImageView
            if (selectedImageView.tag == targetLetter) {
                // Correct
                pageCallback.loadSvgImage(correctImagePathUri, selectedImageView)
            } else {
                pageCallback.loadSvgImage(incorrectImagePathUri, selectedImageView)
            }
        }

        // Frame layout
        val flWidth = resources.displayMetrics.widthPixels
        val flHeight = resources.displayMetrics.heightPixels

        // New dynamic option
        optionHandler = Handler(Looper.getMainLooper())
        optionObject = object:Runnable {
            override fun run() {

                val ivOption = ImageView(view.context)
                val option = content.options[Random.nextInt(content.options.size)]
                ivOption.tag = option.de
                val pathUri= pageCallback.getImageUri(option.imagePath)
                pageCallback.loadSvgImage(pathUri, ivOption)

                val layoutParams = FrameLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT)
                layoutParams.width = (flWidth * 0.15).toInt()
                layoutParams.height = (flHeight * 0.15).toInt()
                layoutParams.leftMargin = (Random.nextInt(layoutParams.width, flListenDynamicOptions.width - layoutParams.width))
                layoutParams.topMargin = (Random.nextInt(layoutParams.height, flListenDynamicOptions.height - layoutParams.height))
                ivOption.layoutParams = layoutParams

                ivOption.setOnClickListener(actionListener)
                flListenDynamicOptions.addView(ivOption)
                visibleOptions.add(option)

                // Remove new object in DYNAMIC_OPTION_LIFE_TIME_ML
                removeHandler = Handler(Looper.getMainLooper())
                removeObject = Runnable {
                    if (flListenDynamicOptions!=null) {
                        flListenDynamicOptions.removeView(ivOption)
                        visibleOptions.remove(option)
                    }
                }
                removeHandler.postDelayed(removeObject, DYNAMIC_OPTION_LIFE_TIME_ML)

                // Create new option
                optionHandler.postDelayed(this, TIME_BETWEEN_DYNAMIC_OPTIONS_ML)
            }
        }
        // Create first option
        optionHandler.postDelayed(optionObject, TIME_BETWEEN_DYNAMIC_OPTIONS_ML)


        // Change target letter
        targetHandler = Handler(Looper.getMainLooper())
        targetObject = object:Runnable {
            override fun run() {

                val targetLetterOld = targetLetter
                var option = content.options.first() // default
                while (targetLetterOld == targetLetter || option == visibleOptions.first()) {
                    option = visibleOptions[Random.nextInt(visibleOptions.size)]
                    targetLetter = option.de
                }

                mpSingle = MediaPlayer.create(view.context, pageCallback.getAudioUri(option.audioPath))
                mpSingle.setOnCompletionListener {
                    mpSingle.release()
                }
                mpSingle.start()

                // Create new target
                targetHandler.postDelayed(this, TIME_UNTIL_NEW_TARGET)
            }
        }
        // Create first target
        targetHandler.postDelayed(targetObject, TIME_UNTIL_NEW_TARGET)
    }

    private fun stop(view: View) {
        buStop.isEnabled = false
        buStop.setBackgroundColor(ContextCompat.getColor(view.context, R.color.colorDarkGray))

        visibleOptions.clear()
        flListenDynamicOptions?.removeAllViews()
        end()

        buStart.setBackgroundColor(ContextCompat.getColor(view.context, levelColor))
        buStart.isEnabled = true
    }

    override fun onPause() {
        super.onPause()
        stop(fragmentView)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        end()
    }

    private fun end() {
        if (this::mpSingle.isInitialized) {
            mpSingle.release()
        }
        if (this::removeHandler.isInitialized) {
            removeHandler.removeCallbacksAndMessages(null)
        }
        if (this::optionHandler.isInitialized) {
            optionHandler.removeCallbacksAndMessages(null)
        }
        if (this::targetHandler.isInitialized) {
            targetHandler.removeCallbacksAndMessages(null)
        }
    }

    companion object {

        private const val ARG_LEVEL = "level"
        private const val ARG_PAGE = "page"

        const val DYNAMIC_OPTION_LIFE_TIME_ML = 8000L
        const val TIME_BETWEEN_DYNAMIC_OPTIONS_ML = 1000L
        const val TIME_UNTIL_NEW_TARGET = 4000L


        @JvmStatic
        fun newInstance(level: String, page: Page) =
            ListenDynamicOptionsFragment().apply {
                arguments = Bundle().apply {

                    putString(ARG_LEVEL, level)
                    putSerializable(ARG_PAGE, page)

                }
            }
    }

}