/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.page.fragments.oralcomprehension

import android.app.AlertDialog
import android.content.res.ColorStateList
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RadioButton
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.jbo.vga.R
import com.jbo.vga.extensions.levelColor
import com.jbo.vga.extensions.sendFeedbackEmail
import com.jbo.vga.model.Content
import com.jbo.vga.model.Page
import com.jbo.vga.page.PageListener
import kotlinx.android.synthetic.main.fragment_oral_comprehension.*
import kotlinx.android.synthetic.main.fragment_oral_comprehension.view.*


class OralComprehensionFragment: Fragment() {

    // Page interface
    private lateinit var pageCallback: PageListener
    fun setPageListener(pageCallback: PageListener) {
        this.pageCallback = pageCallback
    }

    private lateinit var level: String
    private lateinit var page: Page
    private lateinit var currentContent : Content
    private var levelColor = R.color.colorDarkGray // default
    private var currentContentIndex = 0
    private var selectedOptionsCount = 0
    private var selectedOptions = mutableListOf<Int>()

    private var feedBackText: String = ""

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        arguments?.let {
            level = it.getString(ARG_LEVEL).toString()
            levelColor = levelColor(level)
            page = it.getSerializable(ARG_PAGE) as Page
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        val fragmentView = inflater.inflate(R.layout.fragment_oral_comprehension, container, false)
        fragmentView.buFeedback.setBackgroundColor(ContextCompat.getColor(fragmentView.context, levelColor))
        return fragmentView
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // The first content corresponds to the audio
        currentContent = page.contents[currentContentIndex]
        pageCallback.addAudioFragment(level, currentContent.audioPath)

        nextContent(view, true)

        // Set focus change listener
        clOralComprehension.onFocusChangeListener = View.OnFocusChangeListener { it, focused ->
            if (focused) {
                // Hide soft keyboard when edit text lost focus
                pageCallback.onHideSoftKeyboard(it)
            }
        }

        buFeedback.setOnClickListener {

            if (etText.length() > 0) {
                feedBackText+= view.tvContent2.text.toString() + "\n\n" + etText.text + "\n\n"

                // Last content? -> send per email
                if (currentContentIndex==page.contents.size-1) {
                    sendFeedbackEmail(view.context, pageCallback.getFeedbackEmailSubject(level), feedBackText, resources.getText(R.string.email_client).toString())
                } else {
                    nextContent(view, false)
                }
            } else {
                val info = AlertDialog.Builder(view.context)
                info.setMessage(resources.getText(R.string.empty_text))
                info.setPositiveButton("Ok") { _, _ ->}
                info.show()
            }
        }
    }

    private fun nextContent(view: View, firstQuestion: Boolean) {

        // Disable options to avoid user actions until next content is loaded
        for (optionRadioButtons in view.rgOptions.touchables) {
            optionRadioButtons.isEnabled = false
        }

        val timeUntilNextContent = if (firstQuestion) 0 else TIME_UNTIL_NEXT_CONTENT_ML
        Handler(Looper.getMainLooper()).postDelayed({

            // Delete last content options
            view.rgOptions.removeAllViews()
            selectedOptionsCount = 0
            selectedOptions.clear()

            currentContentIndex++

            // Finished?
            if (currentContentIndex<page.contents.size) {

                // No: load next content
                currentContent = page.contents[currentContentIndex]

                when (currentContent.type) {
                    CONTENT_TYPE_CORRECT_OPTION -> correctOptionContent(view, currentContent)
                    CONTENT_TYPE_TEXT_QUESTION -> textQuestionContent(view)
                    else -> pageCallback.onContentFinished() // Return to index
                }
            } else {
                // Return to index
                pageCallback.onContentFinished()
            }

        }, timeUntilNextContent)

    }

    private fun correctOptionContent(view: View, content: Content) {

        view.tvContent?.text = currentContent.de

        val actionListener = View.OnClickListener{ selectedOption ->

            val selectedRadioButton = selectedOption as RadioButton

            if (!selectedOptions.contains(selectedOption.id)) {

                selectedOptionsCount++
                selectedOptions.add(selectedOption.id)

                // Correct answer?
                if (content.correctOptionSequence.contains(selectedOption.id.toString(), ignoreCase = true)) {
                    selectedRadioButton.setTextColor(ContextCompat.getColor(view.context, R.color.correctAnswerColor))
                    nextContent(view, false)
                } else {
                    selectedRadioButton.setTextColor(ContextCompat.getColor(view.context, R.color.incorrectAnswerColor))
                    // Next content?
                    if (selectedOptionsCount == currentContent.options.size) {
                        nextContent(view, false)
                    }
                }
            }
        }

        for ((index, option) in content.options.withIndex()) {

            // Inflate option button used to create the options dynamically
            val rbOption =  LayoutInflater.from(view.context).inflate(R.layout.radio_button_option, view.rgOptions, false) as RadioButton
            rbOption.id = index + 1 // Adaptation of 0 beginning
            rbOption.text = option.de
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                rbOption.buttonTintList = ColorStateList.valueOf(ContextCompat.getColor(view.context, levelColor))
            }
            rbOption.setOnClickListener(actionListener)

            view.rgOptions.addView(rbOption)
        }
    }

    private fun textQuestionContent(view: View) {
        view.tvContent.visibility = View.GONE
        view.tvContent2.visibility = View.VISIBLE
        view.tvContent2?.text = currentContent.de
        view.etText.visibility = View.VISIBLE
        view.etText.setText("")
        view.buFeedback.visibility = View.VISIBLE

        // When last content change buFeedback text from Weiter to Feedback
        if (currentContentIndex==page.contents.size-1) {
            buFeedback.text = resources.getText(R.string.feedback)
        }
    }

    companion object {

        private const val ARG_LEVEL = "level"
        private const val ARG_PAGE = "page"

        const val CONTENT_TYPE_CORRECT_OPTION = "correct_option"
        const val CONTENT_TYPE_TEXT_QUESTION = "text_question"

        const val TIME_UNTIL_NEXT_CONTENT_ML: Long = 1000

        @JvmStatic
        fun newInstance(level: String, page: Page) =
            OralComprehensionFragment().apply {
                arguments = Bundle().apply {

                    putString(ARG_LEVEL, level)
                    putSerializable(ARG_PAGE, page)

                }
            }
    }

}