/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.registration

import com.jbo.vga.model.RemoteDataSource
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.annotations.NonNull
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.observers.DisposableObserver
import io.reactivex.schedulers.Schedulers

class RegistrationPresenter(private var viewInterface: RegistrationContract.ViewInterface, private var remoteDataSource: RemoteDataSource) : RegistrationContract.PresenterInterface {

    private val compositeDisposable = CompositeDisposable()

    /// CALL RESULTS ///
    private val createAccountObservable: (String, String) -> Observable<Boolean> = { email, password -> remoteDataSource.createAccount(email, password) }
    private val createAccountObserver: DisposableObserver<Boolean>
        get() = object : DisposableObserver<Boolean>() {

            override fun onNext(accountCreated: Boolean) {
                if (accountCreated) {
                    viewInterface.hideProgress()
                    viewInterface.registeredSuccessfully()
                }
            }
            override fun onError(@NonNull e: Throwable) {
                viewInterface.hideProgress()
                viewInterface.displayError(e.message.toString())
            }
            override fun onComplete() {
                //
            }
        }

    /// CALLS ///
    override fun createAccount(email: String, password: String) {
        val createAccountDisposable = createAccountObservable(email, password)
            // Subscribe on a background thread
            .subscribeOn(Schedulers.io())
            // Observe on the main thread
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(createAccountObserver)

        compositeDisposable.add(createAccountDisposable)
        viewInterface.showProgress()
    }

    override fun stop() {
        compositeDisposable.clear()
    }

}