/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.didacticunit

import com.jbo.vga.BaseTest
import com.jbo.vga.RxImmediateSchedulerRule
import com.jbo.vga.extensions.buildCachePath
import com.jbo.vga.extensions.buildDidacticUnitRemotePath
import com.jbo.vga.model.Page
import com.jbo.vga.model.RemoteDataSource
import com.jbo.vga.model.RemoteFileSource
import com.nhaarman.mockito_kotlin.times
import io.reactivex.Observable
import org.junit.Before
import org.junit.Rule
import org.junit.Test
import org.junit.runner.RunWith
import org.mockito.Mock
import org.mockito.Mockito
import org.mockito.junit.MockitoJUnitRunner
import java.io.File


@RunWith(MockitoJUnitRunner::class)
class DidacticUnitTests: BaseTest() {

    @Rule
    @JvmField
    var testSchedulerRule = RxImmediateSchedulerRule()

    @Mock
    private lateinit var mockActivity: DidacticUnitContract.ViewInterface

    @Mock
    private lateinit var mockRemoteDataSource: RemoteDataSource

    @Mock
    private lateinit var mockRemoteFileSource: RemoteFileSource


    private lateinit var didacticUnitPresenter: DidacticUnitPresenter
    private lateinit var level: String
    private var didacticUnitNumber: Int = -1
    private lateinit var cacheDirectory: File
    private lateinit var didacticUnitRemotePath: String
    private lateinit var didacticUnitCachePath: String
    private var didacticUnitResourceList = emptyList<String>()

    @Before
    fun setUp() {

        didacticUnitPresenter = DidacticUnitPresenter(
            viewInterface = mockActivity,
            remoteDataSource = mockRemoteDataSource,
            remoteFileSource = mockRemoteFileSource
        )

        level = "a1"
        didacticUnitNumber = 1
        cacheDirectory = File("/cache")
        didacticUnitRemotePath = ""
        didacticUnitCachePath = ""
    }

    @Test
    fun testLoadDidacticUnit() {
        //Set up
        Mockito.doReturn(Observable.just(dummyPageList)).`when`(mockRemoteDataSource).getDidacticUnitPages(level, didacticUnitNumber)

        //Invoke
        didacticUnitPresenter.loadDidacticUnit(level, didacticUnitNumber)

        //Assert
        Mockito.verify(mockRemoteDataSource).getDidacticUnitPages(level, didacticUnitNumber)
        Mockito.verify(mockActivity).openDidacticUnit(dummyPageList)
    }

    @Test
    fun testPostSynchronizeResources() {
        //Set up
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteFileSource).postSynchronizeResources(didacticUnitCachePath, didacticUnitResourceList)

        //Invoke
        didacticUnitPresenter.postSynchronizeResources()

        //Assert
        Mockito.verify(mockActivity).didacticUnitSynchronizationResult(true)
    }

    @Test
    fun testSynchronizeResource() {
        //Set up
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteFileSource).synchronizeResource(dummyDidacticUnitFiles.first(), didacticUnitCachePath, didacticUnitRemotePath)

        //Invoke
        didacticUnitPresenter.synchronizeResource(dummyDidacticUnitFiles.first())

        //Assert
        Mockito.verify(mockRemoteFileSource).synchronizeResource(dummyDidacticUnitFiles.first(), didacticUnitCachePath, didacticUnitRemotePath)
    }


    @Test
    fun testSynchronizeResources() {

        //Set up
        didacticUnitCachePath = buildCachePath(cacheDirectory, level, didacticUnitNumber.toString())
        didacticUnitRemotePath = buildDidacticUnitRemotePath(level, didacticUnitNumber.toString())
        didacticUnitResourceList = dummyDidacticUnitFiles
        // getDidacticUnitResourcesList
        Mockito.doReturn(Observable.just(dummyDidacticUnitFiles)).`when`(mockRemoteFileSource).getDidacticUnitResourcesList(didacticUnitRemotePath)
        // synchronizeResource
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteFileSource).synchronizeResource(dummyDidacticUnitFiles[0], didacticUnitCachePath, didacticUnitRemotePath)
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteFileSource).synchronizeResource(dummyDidacticUnitFiles[1], didacticUnitCachePath, didacticUnitRemotePath)
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteFileSource).synchronizeResource(dummyDidacticUnitFiles[2], didacticUnitCachePath, didacticUnitRemotePath)
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteFileSource).synchronizeResource(dummyDidacticUnitFiles[3], didacticUnitCachePath, didacticUnitRemotePath)
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteFileSource).synchronizeResource(dummyDidacticUnitFiles[4], didacticUnitCachePath, didacticUnitRemotePath)
        // postSynchronizeResources
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteFileSource).postSynchronizeResources(didacticUnitCachePath, didacticUnitResourceList)

        //Invoke
        didacticUnitPresenter.synchronizeResources(level, didacticUnitNumber, cacheDirectory)

        //Assert
        Mockito.verify(mockRemoteFileSource, times(1)).getDidacticUnitResourcesList(didacticUnitRemotePath)
        for (resource in dummyDidacticUnitFiles) {
            Mockito.verify(mockRemoteFileSource).synchronizeResource(resource, didacticUnitCachePath, didacticUnitRemotePath)
        }
        Mockito.verify(mockActivity, times(1)).didacticUnitSynchronizationResult(true)
        Mockito.verify(mockRemoteFileSource, times(1)).postSynchronizeResources(didacticUnitCachePath, didacticUnitResourceList)
    }



    //
    // Helper functions
    //
    private val dummyDidacticUnitFiles: ArrayList<String> = arrayListOf("file1", "file2", "file3", "file4", "file5")

    private val dummyPageList: ArrayList<Page>
        get() {
            val dummyPageList = ArrayList<Page>()
            dummyPageList.add(
                Page(
                    "Title 01 de",
                    "Title 01 es",
                    "exercise type 01",
                    "Statement 01 de",
                    "Statement 01 es"
                )
            )
            dummyPageList.add(
                Page(
                    "Title 02 de",
                    "Title 02 es",
                    "exercise type 02",
                    "Statement 02 de",
                    "Statement 02 es"
                )
            )
            dummyPageList.add(
                Page(
                    "Title 03 de",
                    "Title 03 es",
                    "exercise type 03",
                    "Statement 03 de",
                    "Statement 03 es"
                )
            )
            dummyPageList.add(
                Page(
                    "Title 04 de",
                    "Title 04 es",
                    "exercise type 04",
                    "Statement 04 de",
                    "Statement 04 es"
                )
            )
            dummyPageList.add(
                Page(
                    "Title 05 de",
                    "Title 05 es",
                    "exercise type 05",
                    "Statement 05 de",
                    "Statement 05 es"
                )
            )

            return dummyPageList
        }

}